-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt
local M = {}

local afterfiretime --prevents infinite afterfire
local afterfirecooldown
local afterfireEnabled


local function onInit()
    electrics.values['afterFireDetector'] = 0
	afterfiretime = 0
	afterfirecooldown = 0
	afterfireEnabled = true
end


local function updateGFX(dt)
	--this should disable afterfire on electric motor	
	if not powertrain.getDevice("mainEngine") then return end
	--this should prevent errors when you don't have an engine or transmission installed
	if not electrics.values['gear_M'] then return end
	if not electrics.values['throttle_input'] then return end
	if not electrics.values['engineLoad'] then return end
	if not electrics.values['rpm'] then return end
	
	--disable everything if the engine has no rev limiter
	if powertrain.getDevice("mainEngine").hasRevLimiter == false then return end
	
    local afterfire = 0.5
	--disable afterfire in neutral without revving (that creates engine load but not fire)
	if electrics.values['gear_M'] == 0 and electrics.values['throttle_input'] == 0 then afterfire = 0
	--calculate afterfire effect based on load and throttle input
	--afterfire will work if the result is between 0 and 1
	else 
		afterfire = electrics.values['engineLoad'] + electrics.values['throttle_input'] 
	end
	
	local maxEngineRPM = powertrain.getDevice("mainEngine").revLimiterRPM or 8000
	--disable afterfire if we are holding the gas but the RPM is not close to maxed out
	if electrics.values['rpm']+100 < maxEngineRPM then afterfire = 0 end
	--but always enable when maxed out
	if electrics.values['rpm'] - maxEngineRPM > 10 then 
		afterfire = 0.5 
	end
	
	--popcorn bang
	if powertrain.getDevice("mainEngine").instantEngineLoad <= 0 then 	
		if afterfireEnabled then afterfire = 0.5 end
		afterfiretime = afterfiretime + dt
		if afterfiretime > 0.1 then
			afterfireEnabled = false
			afterfire = 0
			afterfiretime = 0
			afterfirecooldown = 0.1
			return
		end
	end
	if afterfirecooldown > 0 then
		afterfirecooldown = afterfirecooldown - dt
	else
		afterfirecooldown = 0
		afterfireEnabled = true
	end
	
    electrics.values['afterFireDetector'] = afterfire
end


-- public interface
M.onInit      = onInit
M.onReset     = onInit
M.updateGFX = updateGFX

return M