-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}
M.type = "auxiliary"
M.relevantDevice = nil

local controlDisabled = false
local steeringDisabled = false

local axisBeams = nil
local breakBeams = nil

local handbrakeInput = 0
local shifterInput = 0
local lightsInput = 0
local starterInput = true

local handbrakeTimer = 0
local shifterTimer = 0
local lightsTimer = 0
local starterTimer = 0

local desiredArmRotationR = 0
local desiredArmRotationL = 0
local desiredHandRotationR = 0
local desiredHandRotationL = 0
local desiredLegsRotation = 0
local desiredHeadRotation = 0
local desiredTorsoRotation = 0


local function ragdoll()

  for _, b in pairs (axisBeams) do
    obj:breakBeam(b.cid)
  end
  controlDisabled = true
  
end


local function resetDesiredRotation()

  desiredArmRotationR = 0
  desiredArmRotationL = 0
  desiredHandRotationR = 0
  desiredHandRotationL = 0
  desiredLegsRotation = 0
  desiredHeadRotation = 0
  desiredTorsoRotation = 0

end


local function updateGFX(dt)
      
   if controlDisabled then return end

   --ragdoll when body part broken off
   for _, b in pairs (axisBeams) do
     if obj:beamIsBroken(b.cid) then
	   ragdoll()
	   return
	 end
   end
   
   --disable steering when break beam broken
   for _, b in pairs (breakBeams) do
     if obj:beamIsBroken(b.cid) then
	   steeringDisabled = true
	   if b.name == "head" then ragdoll() end --handle decapitation
	 end
   end
   
   
   -----CAR CONTROL, LOWER PARTS HAVE HIGHER PRIORITY-----
   
   --required electrics
   if not electrics.values['parkingbrake_input'] then return end
   if not electrics.values['steering_input'] then return end
   if not electrics.values['shifterX'] then return end
   if not electrics.values['signal_left_input'] then return end
   if not electrics.values['signal_right_input'] then return end
   if not electrics.values['doorFL_input'] then return end
   if not electrics.values['doorFR_input'] then return end
   if not electrics.values['reverse'] then return end
   if not electrics.values['horn'] then return end
   if not electrics.values['lights_state'] then return end
   if not electrics.values['fire_a'] then return end
   if not electrics.values['fire_b'] then return end
   
   
   --first reset all desired rotation to 0
   resetDesiredRotation()
  
   --check inputs and set timers based on them
   if electrics.values.parkingbrake_input ~= handbrakeInput then handbrakeTimer = 0.2 end
   if electrics.values.shifterX 		  ~= shifterInput   then shifterTimer   = 0.5 end
   if electrics.values.lights_state 	  ~= lightsInput    then lightsTimer    = 0.3 end
   
   --starter and shutoff checks if engine was running the frame before
   if electrics.values.running == true and starterInput == false then starterTimer = 1.0 end
   if electrics.values.running == false and starterInput == true then starterTimer = 0.2 end
    
   --while timers are active, set desired rotation
   
   --steering dependent
   if not steeringDisabled then
   
	   --steering: arms, hands
	   if electrics.values.steering_input > 0 then 
		 if electrics.values.steering_input < 0.5 then
		   desiredArmRotationL = math.sqrt(electrics.values.steering_input) * 0.005
		   desiredHandRotationL = electrics.values.steering_input * -0.01
		 else
		   desiredArmRotationL = math.sqrt(electrics.values.steering_input-0.5) * 0.005
		   desiredHandRotationL = (electrics.values.steering_input+0.5) * -0.01
		 end
		 desiredArmRotationR = math.sqrt(electrics.values.steering_input) * -0.005
		 desiredHandRotationR = electrics.values.steering_input * 0.01
	   elseif electrics.values.steering_input < 0 then 
		 if electrics.values.steering_input > -0.5 then
		   desiredArmRotationR = math.sqrt(-electrics.values.steering_input) * 0.005
		   desiredHandRotationR = electrics.values.steering_input * -0.01
		 else
		   desiredArmRotationR = math.sqrt(-electrics.values.steering_input-0.5) * 0.005
		   desiredHandRotationR = (electrics.values.steering_input-0.5) * -0.01
		 end
		 desiredArmRotationL = math.sqrt(-electrics.values.steering_input) * -0.01
		 desiredHandRotationL = electrics.values.steering_input * -0.01
	   end
   
	   --shifter, one time: right arm, legs, torso
	   if shifterTimer > 0 then   
		 shifterInput = electrics.values.shifterX
		 shifterTimer = shifterTimer - dt
		 if electrics.values.shifterX == 0.6 then
		   desiredArmRotationR = -0.01
		 else
		   desiredArmRotationR = -0.02
		   
		   if electrics.values.shifterX < 0.6 then 
		     --manual transmission only - lean to shift if stick is far away
		     if(not electrics.values.gear_A or electrics.values.gear_A==0) then
			   desiredArmRotationR = 0.002
			   desiredLegsRotation = 0.2
			 --automatic
			 else desiredArmRotationR = -0.01 end
		   end
		 end
		 
		 desiredTorsoRotation = 0.005
		 
	   end
   end
   
   --reaction to fire: arms, legs, hands
   if electrics.values.fire_a == 1 then
     desiredArmRotationR = 0.025
	 desiredArmRotationL = 0.020
	 desiredHandRotationL = -0.01
	 desiredHandRotationR = 0.01
	 desiredLegsRotation = 0.2
   end
   if electrics.values.fire_b == 1 then
     desiredArmRotationR = 0.020
	 desiredArmRotationL = 0.025
	 desiredHandRotationL = 0.01
	 desiredHandRotationR = -0.01
	 desiredLegsRotation = 0.2
   end
   
   --handbrake, one time: right arm, right hand
   if handbrakeTimer > 0 then
     desiredArmRotationR = -0.025
	 desiredHandRotationR = -0.01
	 desiredTorsoRotation = 0.01
	 handbrakeInput = electrics.values.parkingbrake_input
	 handbrakeTimer = handbrakeTimer - dt
   end
   
   --starter, one time: right hand
   if starterTimer > 0 then
     desiredHandRotationR = 0.015
	 starterTimer = starterTimer - dt
   end
   
   --lights: left hand
   if lightsTimer > 0 then
     desiredHandRotationL = 0.015 * (electrics.values.lights_state - 0.5)
	 lightsTimer = lightsTimer - dt
   end
   
   --turn signals: head, left hand
   if electrics.values.signal_left_input == 1 then
     desiredHandRotationL = 0.01
	 desiredHeadRotation = 0.01
   end  
   if electrics.values.signal_right_input == 1 then
     desiredHandRotationL = -0.01
	 desiredHeadRotation = -0.01
   end
   
   --doors: head, arms, torso
   if electrics.values.doorFL_input == 1 then
     desiredArmRotationL = -0.025
	 desiredHeadRotation = 0.025
	 desiredTorsoRotation = -0.005
   end  
   if electrics.values.doorFR_input == 1 then
     desiredArmRotationR = -0.025
	 desiredHeadRotation = -0.025
	 desiredTorsoRotation = 0.005
   end
   
   --reverse looking into mirror: head, torso
   if electrics.values.reverse == 1 and electrics.values.running then
     local steer = electrics.values.steering_input + 0.01
	 desiredHeadRotation = 0.02 * fsign(steer) --Use the sign of the steering input to know the sign of output
	 desiredTorsoRotation = -0.0025 * fsign(steer)
   end  
   
   --horn: left arm, legs, torso
   if electrics.values.horn == 1 then
	 desiredArmRotationL = 0.0025
	 desiredLegsRotation = 0.2
	 desiredTorsoRotation = 0.0075
   end  
   
   --get current car state for next frame
   handbrakeInput = electrics.values.parkingbrake_input
   shifterInput   = electrics.values.shifterX
   lightsInput    = electrics.values.lights_state
   starterInput   = electrics.values.running
   
   --add head input
   desiredHeadRotation = desiredHeadRotation - electrics.values.head_input
   
   --proceed with actual animation
   electrics.values.arm_R = desiredArmRotationR
   electrics.values.arm_L = desiredArmRotationL
   electrics.values.hand_R = desiredHandRotationR
   electrics.values.hand_L = desiredHandRotationL 
   electrics.values.leg_L = desiredLegsRotation
   electrics.values.leg_R = desiredLegsRotation
   electrics.values.head = desiredHeadRotation
   electrics.values.torso = desiredTorsoRotation
   
end


local function reset()
  
  --reset hydro angles
  electrics.values.head = 0
  electrics.values.arm_L = 0
  electrics.values.arm_R = 0
  electrics.values.leg_L = 0
  electrics.values.leg_R = 0
  electrics.values.hand_L = 0
  electrics.values.hand_R = 0 
  electrics.values.head = 0 
  electrics.values.head_input = 0 
  electrics.values.torso = 0 
  
  resetDesiredRotation()
  
  --reset vehicle input
  handbrakeInput = 0
  shifterInput = 0
  lightsInput = 0
  starterInput = true
  
  --reset timers
  handbrakeTimer = 0
  shiterTimer = 0
  lightsTimer = 0
  starterTimer = 0
  
  --set some default stuff
  controlDisabled = false
  steeringDisabled = false
  
end


local function init(jbeamData)	

  --for handbrake behavior
  electrics.values.handbrake_delay_time = 0.3
  
  --init axis beams
  axisBeams = {}
  if jbeamData.axisBeams then
    for _, axisBeamName in pairs (jbeamData.axisBeams) do
	  for _, b in pairs(v.data.beams) do
	    if b.name and string.match(b.name, axisBeamName) then
		  table.insert(axisBeams, b)
		end
	  end
	end
  end
  
  --init break beams
  breakBeams = {}
  if jbeamData.breakBeams then
    for _, breakBeamName in pairs (jbeamData.breakBeams) do
	  for _, b in pairs(v.data.beams) do
	    if b.name and string.match(b.name, breakBeamName) then
		  table.insert(breakBeams, b)
		end
	  end
	end
  end
	
  --reset electrics
  reset()
  
end


local function rotHead(value)
  electrics.values.head_input = 0.05 * value
end


--public interface
M.init = init
M.reset = reset
M.ragdoll = ragdoll
M.updateGFX = updateGFX
M.rotHead = rotHead

return M
