-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}

--main variables
local legogun_currentbullet_num
local legogun_timeBetweenBullets
local legogun_canShoot --boolean
local legogun_bulletsLeft
local legogun_bulletsLeftPercentage
local legogun_breakBeam
local legogun_isBroken --boolean
local legogun_currentBeltSection_num
local legogun_requiredBeltSectionLength
local legogun_beltDetachBeam
local legogun_isBeltDetached --boolean

--from jbeamData
local legogun_ammo
local legogun_shoot_delay
local legogun_guntype
local legogun_bulletMass
local legogun_decreaseAmmoWeightOnShooting --boolean
local legogun_bulletMassInAmmo
local legogun_breakable --boolean
local legogun_breakBeamName
local legogun_hasBelt --boolean
local legogun_bulletsPerBeltSection
local legogun_beltDetachable --boolean
local legogun_beltDetachBeamName
local legogun_simplifiedMode --boolean
local legogun_customFireElectrics
local legogun_IDproperty

--variables for nodes
local legogun_gunSoundNode --gun sounds location
local legogun_fireParticleNodeInner --shoot particles location
local legogun_fireParticleNodeOuter --shoot particles direction
local legogun_ammoWeightStorageNode --stores the weight of the gun's ammo, more useful for flamethrower since the machine gun bullets are light

--variables for names of the nodes from JbeamData
local legogun_gunSoundNodeID
local legogun_fireParticleNodeInnerID
local legogun_fireParticleNodeOuterID
local legogun_ammoWeightStorageNodeID

local function init(jbeamData)

   legogun_currentbullet_num = 1
   for k, node in pairs (v.data.nodes) do
	--reset particle properties
	node.horizontalVelocity = nil
	node.newHorizontalVelocity = nil
	node.exploded = nil
   end


  --get jbeamData
  legogun_ammo = jbeamData.legogun_ammo or 100
  legogun_shoot_delay = jbeamData.legogun_shoot_delay or 0.075
  legogun_guntype = jbeamData.legogun_guntype or "Machinegun"
  legogun_bulletMass = jbeamData.legogun_bulletMass or 2.5
  legogun_gunSoundNodeID = jbeamData.legogun_gunSoundNode or "legogun2"
  legogun_decreaseAmmoWeightOnShooting = jbeamData.legogun_decreaseAmmoWeightOnShooting or false
  legogun_fireParticleNodeInnerID = jbeamData.legogun_fireParticleNodeInner or "legogun2"
  legogun_fireParticleNodeOuterID = jbeamData.legogun_fireParticleNodeOuter or "legogun1"
  legogun_ammoWeightStorageNodeID = jbeamData.legogun_ammoWeightStorageNode or "legogun_ammo"
  legogun_bulletMassInAmmo = jbeamData.legogun_bulletMassInAmmo or 2.5
  legogun_breakable = jbeamData.legogun_breakable or false
  legogun_breakBeamName = jbeamData.legogun_breakBeamName or "legogun_broken"
  legogun_hasBelt = jbeamData.legogun_hasBelt or false
  legogun_bulletsPerBeltSection = jbeamData.legogun_bulletsPerBeltSection or 10
  legogun_beltDetachable = jbeamData.legogun_beltDetachable or false
  legogun_beltDetachBeamName = jbeamData.legogun_beltDetachBeamName or "legogun_belt_detach"
  legogun_hasSecondaryGun = jbeamData.legogun_hasSecondaryGun or false
  legogun_gunSoundNodeID2 = jbeamData.legogun_gunSoundNode2 or "legogun2r"
  legogun_fireParticleNodeInnerID2 = jbeamData.legogun_fireParticleNodeInner2 or "legogun2r"
  legogun_fireParticleNodeOuterID2 = jbeamData.legogun_fireParticleNodeOuter2 or "legogun1r"
  legogun_simplifiedMode = jbeamData.legogun_simplifiedMode or true
  legogun_customFireElectrics = jbeamData.legogun_customFireElectrics or "horn" --some random electrics value to prevent crashes
  legogun_IDproperty = jbeamData.legogun_IDproperty or "legogun_bulletID" --this is needed to support multiple independently shooting guns
  
  --reset electrics values for firing all bullets
  for i=1, legogun_ammo, 1 do
    electrics.values["shootbullet_" .. i] = 0
  end
  
  --reset electrics values for belt control
  for i=1, (legogun_ammo / legogun_bulletsPerBeltSection + 1), 1 do
    electrics.values["legogun_belt_" .. i] = 1
  end
  
  --reset other electrics
  electrics.values['legogun_fire'] = 0
  
  --reset local variables
  legogun_currentbullet_num = 1
  legogun_timeBetweenBullets = 0
  legogun_canShoot = true
  legogun_isBroken = false
  legogun_currentBeltSection_num = 1
  legogun_requiredBeltSectionLength = 1
  legogun_isBeltDetached = false
  
  --assign nodes to variables based on IDs from jbeamData
  for k, node in pairs (v.data.nodes) do
    if node.name == legogun_gunSoundNodeID then legogun_gunSoundNode = k  end
	if node.name == legogun_fireParticleNodeInnerID then legogun_fireParticleNodeInner = k  end
	if node.name == legogun_fireParticleNodeOuterID then legogun_fireParticleNodeOuter = k  end
	--we need the cid here since we will be changing properties of the node itself
	if node.name == legogun_ammoWeightStorageNodeID then legogun_ammoWeightStorageNode = node.cid end
  end
  
  --get break beam cids
  for _,b in pairs(v.data.beams) do
	if b.name == legogun_breakBeamName then legogun_breakBeam = b.cid end
	if b.name == legogun_beltDetachBeamName then legogun_beltDetachBeam = b.cid end
  end
  
  --display GUI message based on gun type (this doesn't work sometimes for some reason)
  if legogun_guntype == "flamethrower" then
    guihooks.message("Flamethrower fuel left: 100%", 5, "AgentY_gun")
  elseif legogun_guntype == "RPG" then
    guihooks.message("Ammo left: " .. legogun_ammo .. ", delay: " .. legogun_shoot_delay .. "s", 5, "AgentY_gun")
  elseif legogun_guntype == "machinegun" then
	guihooks.message("Bullets left (" .. legogun_customFireElectrics .. "): " .. legogun_ammo, 5, "AgentY_gun")
  end
   
end


local function updateGFX(dt) -- ms

  -- HIT PARTICLES (HELP WITH CODE BY AWESOMECARL) --
  if (legogun_guntype == "Machinegun" or legogun_guntype == "RPG") then
	--track horizontal velocity for each fired node
	for k, node in pairs (v.data.nodes) do
	  --check if node is a bullet and then if it has been fired already
      if node[legogun_IDproperty] and node[legogun_IDproperty] <= legogun_currentbullet_num - 1 then 
	  
	    --check if node had already emited particles
		if node.exploded == nil then

		  --mid air particles
		  obj:addParticleByNodesRelative(node.cid, node.cid, 10, 9, 0.02, 2)
	      obj:addParticleByNodesRelative(node.cid, node.cid, 10, 23, 0.02, 2)
	  
	      local velocity = vec3(obj:getNodeVelocityVector(node.cid))		
		  --check if node has just been fired (has no set velocity)
		  if node.newHorizontalVelocity == nil then
		    node.newHorizontalVelocity = (math.abs(velocity.x) + math.abs(velocity.y))
		  else
		    --update velocity, store old one in different variable
		    node.horizontalVelocity = node.newHorizontalVelocity
		    node.newHorizontalVelocity = (math.abs(velocity.x) + math.abs(velocity.y))	  
		    --check if node hit something - velocity has decreased significantly
		    if node.horizontalVelocity - node.newHorizontalVelocity > 0.1 then
		     
			  if (legogun_guntype == "Machinegun") then 		  
				  --add particles once
				  obj:addParticleByNodesRelative(node.cid, node.cid, 12, 1, 0.02, 2)
				  obj:addParticleByNodesRelative(node.cid, node.cid, 15, 61, 0.02, 2)
				  obj:addParticleByNodesRelative(node.cid, node.cid, 10, 62, 0.02, 2)
				  obj:addParticleByNodesRelative(node.cid, node.cid, 20, 63, 0.02, 2)
				  obj:addParticleByNodesRelative(node.cid, node.cid, 8, 64, 0.02, 2)
				  obj:addParticleByNodesRelative(node.cid, node.cid, 12, 65, 0.02, 2)
				  obj:addParticleByNodesRelative(node.cid, node.cid, 10, 6, 0.02, 2)
			  else
			  
			    --particles
				obj:addParticleByNodesRelative(node.cid, node.cid, 5, 49, 1, 10)
				obj:addParticleByNodesRelative(node.cid, node.cid, 5, 29, 1, 10)
				obj:addParticleByNodesRelative(node.cid, node.cid, 5, 37, 1, 10)
								
			  end --both gun types
			  node.exploded = true
			end
			
		  end		  
		end
	  end 	  
    end	

  end

  -- CHECKING IF GUN IS BROKEN --  
  if legogun_breakable then
    if legogun_isBroken then return end --if it's already broken, you can't do anything 
    local legogun_checkIfBroke = obj:beamIsBroken(legogun_breakBeam) --check if it just broke each frame
    if legogun_checkIfBroke then
      guihooks.message(legogun_guntype .. " (" .. legogun_customFireElectrics .. ") broken", 10, "vehicle.damage") --notify the user ONLY ONCE that it just broke
	  legogun_isBroken =  true -- disable gun
      return
    end
  end
  
  -- CHECKING IF BELT IS DETACHED --  
  if legogun_beltDetachable then
	local legogun_checkIfBeltDetached = obj:beamIsBroken(legogun_beltDetachBeam) --check if it just broke each frame
	if legogun_checkIfBeltDetached then
		guihooks.message("Ammo belt detached, no bullets!", 10, "vehicle.damage") --notify the user ONLY ONCE that it just broke
		legogun_isBeltDetached =  true -- will act like it's trying to shoot but has no ammo
	end
  end
  
  -- CHECKING IF WE CAN SHOOT --
  legogun_timeBetweenBullets = legogun_timeBetweenBullets + dt --measure time between firing each bullet
  if legogun_timeBetweenBullets < legogun_shoot_delay then 
    electrics.values[legogun_customFireElectrics] = 0
    return 
  end --can't shoot if timer is not over

  -- DETECTING INPUT --
  if electrics.values.legogun_fire < 0.9 then return end --axis/button is not being pressed down far enough, ignore this event
  if legogun_currentbullet_num > legogun_ammo then return end -- don't shoot if we are out of ammo
  
  -- FIRING THE BULLETS --
  -- shoot bullet nr. legogun_currentbullet_num, but only if the ammo belt is attached
  if legogun_simplifiedMode then --less nodes, no thrusters, works on beam breaking, hopefully less lag
    if not legogun_isBeltDetached then
	  for _,b in pairs(v.data.beams) do
	    --to find the beams we use a little trick: beams are actually arrays, not objects, so we can pass the property name as variable header
	    if b[legogun_IDproperty] == legogun_currentbullet_num then 
		  obj:breakBeam(b.cid)
		end
	  end
	end
  else
	if not legogun_isBeltDetached then electrics.values["shootbullet_" .. legogun_currentbullet_num] = 1 end
  end
  legogun_currentbullet_num = legogun_currentbullet_num + 1 --go to next bullet in line
  legogun_canShoot = false --disable shooting until time passes
  legogun_timeBetweenBullets = 0 --reset timer
  legogun_bulletsLeft = legogun_ammo - legogun_currentbullet_num + 1
  
  --Remove the weight of each node from the ammo storage weight
  if legogun_decreaseAmmoWeightOnShooting == true then 
	local currentAmmoNodeMass = obj:getNodeMass(legogun_ammoWeightStorageNode)
	local AmmoNodeMassAfterShooting = currentAmmoNodeMass - legogun_bulletMassInAmmo
	obj:setNodeMass(legogun_ammoWeightStorageNode, AmmoNodeMassAfterShooting)
  end
  
  --Now we have different behavior for different gun types
  
  if legogun_guntype == "Flamethrower" then
    legogun_bulletsLeftPercentage = legogun_bulletsLeft / legogun_ammo * 100 --display ammo as percentage of fuel left
    guihooks.message("Flamethrower fuel left: " .. legogun_bulletsLeftPercentage .. "%", 5, "AgentY_gun") 
    --play fire sound
    obj:playSFXOnce("event:>Vehicle>Fire>Fire_Ignition", legogun_gunSoundNode, 1, 1)	 
  
  elseif legogun_guntype == "RPG" then
  
	guihooks.message("Ammo left: " .. legogun_bulletsLeft, 5, "AgentY_gun")
	--play fire sound
	obj:playSFXOnce("event:>Special>cannon_fire", legogun_gunSoundNode, 1, 2)
	--show firing particles
	obj:addParticleByNodesRelative(legogun_fireParticleNodeInner, legogun_fireParticleNodeOuter, 5, 49, 1, 10)
	obj:addParticleByNodesRelative(legogun_fireParticleNodeInner, legogun_fireParticleNodeOuter, 5, 29, 1, 10)
	obj:addParticleByNodesRelative(legogun_fireParticleNodeInner, legogun_fireParticleNodeOuter, 5, 37, 1, 10)
  
  elseif (legogun_guntype == "Machinegun") and (legogun_isBeltDetached == false) then
  
	guihooks.message("Bullets left (" .. legogun_customFireElectrics .. "): " .. legogun_bulletsLeft, 5, "AgentY_gun")

		--play fire sound
		obj:playSFXOnce("CrashTestSound", legogun_gunSoundNode, 1, 2)
		--show firing particles
		obj:addParticleByNodesRelative(legogun_fireParticleNodeInner, legogun_fireParticleNodeOuter, 15, 61, 0, 1)
		obj:addParticleByNodesRelative(legogun_fireParticleNodeInner, legogun_fireParticleNodeOuter, 10, 62, 0, 1)
		obj:addParticleByNodesRelative(legogun_fireParticleNodeInner, legogun_fireParticleNodeOuter, 20, 63, 0, 1)
		obj:addParticleByNodesRelative(legogun_fireParticleNodeInner, legogun_fireParticleNodeOuter, 8, 64, 0, 1)
		obj:addParticleByNodesRelative(legogun_fireParticleNodeInner, legogun_fireParticleNodeOuter, 12, 65, 0, 1)
		--show smoke
		obj:addParticleByNodesRelative(legogun_fireParticleNodeInner, legogun_fireParticleNodeOuter, 10, 6, 0, 1)
	
	--custom particles
	electrics.values[legogun_customFireElectrics] = 1
	
	--increase weight of each bullet on the fly
	--this is not very realistic but it's the only way we can deal damage basically
	--it increases bullet energy and keeps momentum because it slows down bullets a bit
	--and if the bullets had this weight from the start, the vehicle wouldn't be able to hold the gun due to the weight
	for k, node in pairs (v.data.nodes) do
      if node[legogun_IDproperty] == legogun_currentbullet_num - 1 then 
	    obj:setNodeMass(node.cid, legogun_bulletMass)
	  end 
    end	
	
	-- BELT ANIMATION --
	if (legogun_hasBelt == true) and (legogun_isBeltDetached == false) then
		--determine which section of the belt we are currently retracing based on the current bullet number
		legogun_currentBeltSection_num = math.floor((legogun_currentbullet_num - 2) / legogun_bulletsPerBeltSection) + 1 --not sure why this works but it does lol 
		--retrace current section by a set amount
		legogun_requiredBeltSectionLength = legogun_requiredBeltSectionLength - (1 / legogun_bulletsPerBeltSection)
		if legogun_requiredBeltSectionLength < 0 then legogun_requiredBeltSectionLength = legogun_requiredBeltSectionLength + 1 end
		electrics.values["legogun_belt_" .. legogun_currentBeltSection_num] = legogun_requiredBeltSectionLength - 1
	end
		
  end
  
end

-- public interface

M.init         = init
M.updateGFX    = updateGFX

return M
