-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}
local ceil = math.ceil
local max = math.max

local function onReset()

  electrics.values['wigwag_L2'] = 0
  electrics.values['wigwag_R2'] = 0
  electrics.values['wigwag_L3'] = 0
  electrics.values['wigwag_R3'] = 0
  electrics.values['wigwag_L4'] = 0
  electrics.values['wigwag_R4'] = 0
  
  electrics.values['cop_EU_BL'] = 0
  electrics.values['cop_EU_BR'] = 0
  electrics.values['cop_EU_S'] = 0
  
end

local function updateGFX(dt)

  local vals = electrics.values
  
  local lightbarActive = vals.lightbar > 0
  local wigwagR3Active = vals.wigwag_R3 == 1
  local wigwagL3Active = vals.wigwag_L3 == 1
  local wigwagR4Active = vals.wigwag_R4 == 1
  local wigwagL4Active = vals.wigwag_L4 == 1
  local lowbeamActive = vals.lowbeam == 1
  local highbeamActive = vals.highbeam == 1
  local lowhighbeamActive = vals.lowhighbeam == 1
  
  vals.fog_wigwag_R = vals.wigwag_R == 1 or vals.fog == 1
  vals.fog_wigwag_L = vals.wigwag_L == 1 or vals.fog == 1
  vals.wigwag_signal_R = vals.signal_right_input == 1 and vals.signal_R or vals.wigwag_R2 == 1 
  vals.wigwag_signal_L = vals.signal_left_input == 1 and vals.signal_L or vals.wigwag_L2 == 1
  vals.lowbeam_wigwag_R = lowbeamActive or wigwagR3Active
  vals.lowbeam_wigwag_L = lowbeamActive or wigwagL3Active
  vals.brake_wigwag_R = vals.brakelights == 1 or vals.wigwag_R3 == 1 
  vals.brake_wigwag_L = vals.brakelights == 1 or vals.wigwag_L3 == 1
  vals.lowhighbeam_wigwag_R = ((lowhighbeamActive and not lightbarActive) or (wigwagR4Active))
  vals.lowhighbeam_wigwag_L = ((lowhighbeamActive and not lightbarActive) or (wigwagL4Active))
  
  vals.cop_EU_BR = vals.signal_right_input == 1 and vals.signal_R or vals.cop_EU_signalR == 1 
  vals.cop_EU_BL = vals.signal_left_input == 1 and vals.signal_L or vals.cop_EU_signalL == 1
  vals.cop_EU_S = (lightbarActive and (highbeamActive or vals.cop_EU_spotlight == 1))
  
end

-- public interface
M.onInit    = onReset
M.onReset   = onReset
M.updateGFX = updateGFX

return M
