-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt
local M = {}

local nitrofiretime
local nitrofirecooldown
local firetime
local firecooldown


local function onInit()
    electrics.values['nitrofire_primary'] = 0
	electrics.values['nitrofire_secondary'] = 0
	electrics.values['fire_a'] = 0
	electrics.values['fire_b'] = 0
	electrics.values['fire_a_fuelcell'] = 0
	electrics.values['fire_b_fuelcell'] = 0
	nitrofiretime = 0
	nitrofirecooldown = 0
	firetime = 0
	firecooldown = 0
end


local function enableFire(dt, fireA, fireB, type_a, type_b)

  firetime = firetime + dt
  if firetime > 0.1 then
	firetime = 0
	firecooldown = 0.1
  end
  if firecooldown > 0 then
	firetime = 0
	firecooldown = firecooldown - dt
	fireA = 0
	fireB = 1
  else
	firecooldown = 0
	fireA = 1
	fireB = 0
  end

  electrics.values[type_a] = fireA
  electrics.values[type_b] = fireB

end


local function updateGFX(dt)

    --disable all fire under water
	for k, _ in pairs(v.data.nodes) do
	  if obj:inWater(k) then
	    electrics.values['nitrofire_primary'] = 0
	    electrics.values['nitrofire_secondary'] = 0
	    electrics.values['fire_a'] = 0
	    electrics.values['fire_b'] = 0
		electrics.values['fire_a_fuelcell'] = 0
	    electrics.values['fire_b_fuelcell'] = 0
	    return 
	  end
	end

	--HANDLE GENERIC FIRE EFFECT--
	
	local fireA = 0
	local fireB = 0
	
	--enable fire on fuel tank break
	for k, b in pairs(v.data.beams) do
	  if b.deformGroup == 'fueltank_break' and obj:beamIsBroken(k) then
		enableFire(dt, fireA, fireB, 'fire_a', 'fire_b')
	  elseif b.deformGroup == 'fuelcell_break' and obj:beamIsBroken(k) then
		enableFire(dt, fireA, fireB, 'fire_a_fuelcell', 'fire_b_fuelcell')
	  end
	end
	
	--enable fire by button (not working for some reason)
	for _, n in pairs(v.data.nodes) do
	  if n.containerBeamBroken then
		enableFire(dt, fireA, fireB, 'fire_a', 'fire_b')
		enableFire(dt, fireA, fireB, 'fire_a_fuelcell', 'fire_b_fuelcell')
	  end
	end


	--HANDLE NITROUS OXIDE FIRE EFFECT--

	--this should prevent errors when you don't have nitrous oxide installed
	if not electrics.values["nitrousOxideActive"] then return end
	
	--this should prevent errors if you somehow have no engine but still have nitrous
	if not electrics.values["engineRunning"] then return end
	
    local nitrofirePrimary = 0
	local nitrofireSecondary = 0

	if electrics.values["nitrousOxideActive"] == 1 and electrics.values["engineRunning"] == 1 then
		nitrofiretime = nitrofiretime + dt
		if nitrofiretime > 0.05 then
			nitrofiretime = 0
			nitrofirecooldown = 0.05
		end
		if nitrofirecooldown > 0 then
			nitrofiretime = 0
			nitrofirecooldown = nitrofirecooldown - dt
			nitrofirePrimary = 0
			nitrofireSecondary = 1
		else
			nitrofirecooldown = 0
			nitrofirePrimary = 1
			nitrofireSecondary = 0
		end
	end	
	
    electrics.values['nitrofire_primary'] = nitrofirePrimary
	electrics.values['nitrofire_secondary'] = nitrofireSecondary
end


-- public interface
M.enableFire  = enableFire
M.onInit      = onInit
M.onReset     = onInit
M.updateGFX = updateGFX

return M